require "behaviours/follow"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/panic"


local MIN_FOLLOW_DIST = 0
local MAX_FOLLOW_DIST = 12
local TARGET_FOLLOW_DIST = 6

local MAX_WANDER_DIST = 3


local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function WhoToFollow(inst) --modification to make chester follow luggage but rincewind be the leader (3' followers do not behave as they should)
	local luggage = TheSim:FindFirstEntityWithTag("luggage")
	if luggage and luggage.components.container:Has("chester_eyebone",1) then
		return luggage
	else
		return inst.components.follower.leader
	end
	
end

local ChesterBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)


function ChesterBrain:OnStart()
    local root = 
    PriorityNode({
        WhileNode( function() return self.inst.components.health.takingfiredamage end, "OnFire", Panic(self.inst)),
        Follow(self.inst, function() return WhoToFollow(self.inst) end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
        FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
        Wander(self.inst, function() return self.inst.components.knownlocations:GetLocation("home") end, MAX_WANDER_DIST),
        
    }, .25)
    self.bt = BT(self.inst, root)
end

return ChesterBrain
