require "behaviours/follow"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/panic"
require "behaviours/chaseandattack"
require "behaviours/doaction"
require "behaviours/leash"
require "behaviours/findlight"
require "behaviours/attackwall"
require "behaviours/standstill"

local MIN_FOLLOW_DIST = 0
local MAX_FOLLOW_DIST = 6
local TARGET_FOLLOW_DIST = 3

local MAX_WANDER_DIST = 3

local MAX_CHASE_TIME = 10
local MAX_CHASE_DIST = 30
local KEEP_CHOPPING_DIST = 15
local SEE_TREE_DIST = 6 --15
local SEE_INSECT_DIST = 10
local LEASH_RETURN_DIST = 10
local LEASH_MAX_DIST = 30
local INSTA_TAKE_DIST = 4

local ValidToInstaTake = 
{
    "meat",
	"flint",
    "beefalowool",
    "butter",   
    "butterflywings",
    "charcoal",
    "deerclops_eyeball ", 
    "feather_crow",
    "feather_robin",
    "feather_robin_winter",   
    "froglegs",
    "gears",
    "goldnugget", 
	"guano",
    "honey",
    "honeycomb",   
    --"horn",
    "houndstooth",
    "marble",
	"livinglog",
    "rocks",
    "log",
    "smallmeat",
    "drumstick",
    "batwing", 
    "nightmarefuel", 
	"nitre",
    "pigskin",
    "silk",
    "slurtleslime",
    "slurtle_shellpieces", 
    "spidergland",
    "spidereggsack",
    --"stinger",   
    --"tentaclespike",
    "tentaclespots",
    "trunk_summer",
	"trunk_winter",
    --"walrus_tusk",
    "foliage", 
    "lureplant_bulb", 
	"thulecite",
    "slurper_pelt",
	--"hat_wizzard",
	"monstermeat",    -- CHEATING
	"pinecone",    -- CHEATING
	--"poop", -- CHEATING
	--"seeds", -- CHEATING
}

if IsDLCEnabled(PORKLAND_DLC) then 
	DLCInstaTake = {
	"vine",
	"teatree_nut",
	}
	for k,v in pairs(DLCInstaTake) do
		table.insert(ValidToInstaTake, v)
	end
elseif IsDLCEnabled(CAPY_DLC) then 
	DLCInstaTake = {
	"coconut",
	"palmleaf",
	"fish",
	"jungletreeseed",
	"bamboo",
	"vine",
	"fish_raw_small",
	}
	for k,v in pairs(DLCInstaTake) do
		table.insert(ValidToInstaTake, v)
	end
elseif IsDLCEnabled(REIGN_OF_GIANTS) then 
	DLCInstaTake = {
	"acorn",
	}
	for k,v in pairs(DLCInstaTake) do
		table.insert(ValidToInstaTake, v)
	end
end


local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function KeepChoppingAction(inst)
    return inst.components.follower.leader and inst.components.follower.leader:GetDistanceSqToInst(inst) <= KEEP_CHOPPING_DIST*KEEP_CHOPPING_DIST
end

local function StartChoppingCondition(inst)
    return inst.components.follower.leader and inst.components.follower.leader.sg:HasStateTag("chopping")
end

local function StartMiningCondition(inst)
    return inst.components.follower.leader and inst.components.follower.leader.sg:HasStateTag("mining")
end

local function StartHackingCondition(inst)
    return inst.components.follower.leader and inst.components.follower.leader.sg:HasStateTag("hacking")
end


local function FindTreeToChopAction(inst)
    local target = FindEntity(inst, SEE_TREE_DIST, function(item) return item.components.workable and item.components.workable.action == ACTIONS.CHOP end)
    if target then
        return BufferedAction(inst, target, ACTIONS.CHOP)
	end
	local target = FindEntity(inst, SEE_TREE_DIST, function(item) return item.components.workable and item:HasTag("stump") and item.components.workable.action == ACTIONS.DIG end)
	if target then
        return BufferedAction(inst, target, ACTIONS.DIG)
	end
end

local function FindRockToMineAction(inst)
	local target_mine = FindEntity(inst, SEE_TREE_DIST, function(item) return item.components.workable and item.components.workable.action == ACTIONS.MINE end)
	--print("rock seen", target_mine, target_mine.stage)
	if target_mine then --need a special case not to mine puddles
		if target_mine.prefab == "rock_ice" and target_mine.stage == "empty" then
		else
			return BufferedAction(inst, target_mine, ACTIONS.MINE)
		end
    end
end

local function FindVineToHackAction(inst)
	--print("Looking for vine")
	local target_vine = FindEntity(inst, SEE_TREE_DIST, function(item) return item.components.hackable and item.components.hackable:CanBeHacked() end)
	if target_vine then 
		--print("vine seen", target_vine, target_vine.stage)
		if target_vine:HasTag("stump") then
		else
			return BufferedAction(inst, target_vine, ACTIONS.HACK)
		end
    end
end


local function PlayerBees(target_insect) --prevents luggage from snapping your worker bees
	--print("I see insect", target_insect.prefab, target_insect)
	if target_insect.prefab == "bee" then 
		local target_home = target_insect.components.knownlocations.locations["home"]
		if not target_home then return false end
		--print(target_home.x, target_home.y, target_home.z, 3)
		local ents = TheSim:FindEntities(target_home.x, target_home.y, target_home.z, 3)
		for k,v in pairs(ents) do
			if v.prefab == "beebox" then 
				--print("Player bee", v.prefab)
				return true 
			end
		end
		if target_insect.components.homeseeker then
			if target_insect.components.homeseeker.home.prefab == "beebox" then return true end
		end
	end
	return false
end

local function SnapAtInsects(inst)
	local target_insect = FindEntity(inst, SEE_INSECT_DIST, function(item) return item.components.workable and item.components.workable.action == ACTIONS.NET end)
	if target_insect then
		if not PlayerBees(target_insect) then
			--if target_insect.components.homeseeker then print("I see insect, and will eat it. ", target_insect.prefab) end --, target_insect.components.homeseeker.home.prefab
			if target_insect.components.inventoryitem then 
				target_insect.components.inventoryitem.canbepickedup = true
				if not target_insect.components.luggagepickupable then
					target_insect:AddComponent("luggagepickupable")
				end
				return BufferedAction(inst, target_insect, ACTIONS.LUGGAGE_PICKUP)
			end
		end
    end
end

local LuggageBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local function ItemIsInList(item, list)
    for k,v in pairs(list) do
        if v == item or k == item then
            return true
        end
    end
end

local function SaveLuggInv(dirty, inst)
	local lugg_contents = {}
	for k,v in pairs(inst.components.container.slots) do
        if v:IsValid() then --only save the valid items
			lugg_contents[k] = v:GetSaveRecord()
		end
    end
	--for k,v in pairs(lugg_contents) do print ("saving", k, v, "<-------") end
	inst.lugg_inv:SetValue("lugg_inv", lugg_contents)
	inst.lugg_inv:Save(dirty)
end

local function TryToInstaTake(inst)
	local instataketarget = nil
    --Get the stuff around you and store it in ents
    local pt = inst:GetPosition()
    --local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, INSTA_TAKE_DIST)  
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, INSTA_TAKE_DIST)  
	
    if not instataketarget then
        for k,item in pairs(ents) do
			--if not item.components.inventoryitem then print("1") end
			--if not item.components.shelf.itemonshelf then print("2") end
			--if not item.components.shelf.cantakeitem then print("3") end
			--if not item:IsOnValidGround() then print("4") end
            --if item.components.shelf and item.components.shelf.itemonshelf and item.components.shelf.cantakeitem and item:IsOnValidGround()
            --and ItemIsInList(item.components.shelf.itemonshelf, ValidToInstaTake) then
			if item:IsOnValidGround() and item.components.inventoryitem and item.components.inventoryitem.canbepickedup and not
                item.components.inventoryitem:IsHeld() and ItemIsInList(item.prefab, ValidToInstaTake) then
                instataketarget = item
				--print("takingitem",item.prefab)
                break
            end
			--if ItemIsInList(item, ValidToInstaTake) then
            --    instataketarget = item
			--	print("INSTAPICK found!")
            --    break
            --end
        end
    end
	
	if instataketarget then
		if not instataketarget.components.luggagepickupable then
			instataketarget:AddComponent("luggagepickupable")
		end
		--print("ready to grab", instataketarget)
        --Alright, yeah! That's - no that's a pretty good job!
		--DoAction(inst, BufferedAction(inst,instataketarget,ACTIONS.PICKUP) )
		--TryToInstaTake
		SaveLuggInv(true, inst)
        return BufferedAction(inst,instataketarget,ACTIONS.LUGGAGE_PICKUP)
    end
	
	
end

local function WallInWay(inst, self)
	local pos = Point(inst.Transform:GetWorldPosition())
	local target_pos = Point(GetPlayer().Transform:GetWorldPosition())
	local dist_sq = distsq(pos, target_pos)
	--print("Is there a wall in the way?")
	if dist_sq > MAX_FOLLOW_DIST*MAX_FOLLOW_DIST then
	 --print("Yes! Fuck that wall!")
	 return true
	end
end

local function IsPlayerUsingItem(inst, item)
	local player = GetPlayer()
	--print("this is the item", player.components.inventory:IsItemEquipped(item))
	return player.components.inventory:IsItemEquipped(item)
end

local function FindTheHatAndStealIt(inst)
	local act = BufferedAction(inst, IsPlayerUsingItem(inst, "hat_wizzard"), ACTIONS.STEAL)
	act.validfn = function() return true end
	return act
end

local function KillChester(inst)
	--inst.components.combat.defaultdamage = 200000
	local has_eyebone, nr_eyebone = inst.components.container:Has("chester_eyebone",1)
	local has_fishbone, nr_fishbone = inst.components.container:Has("packim_fishbone",1)
	--print("eyebone", has_eyebone) --debug
	--has_eyebone = true   --debug
	return FindEntity(inst, SEE_INSECT_DIST, function(guy) 
		return guy:HasTag("chester") and inst.components.combat:CanTarget(guy) and not (has_eyebone or has_fishbone)
    end)
end



function LuggageBrain:OnStart()
    local root = 
    PriorityNode({
		-- IfNode(function() return AttackCondition(self.inst) end, "attack", 
                -- WhileNode(function() return KeepChoppingAction(self.inst) end, "keep chopping",
                    -- LoopNode{ 
                            -- DoAction(self.inst, FindTreeToChopAction )})),
		ChaseAndAttack(self.inst, MAX_CHASE_TIME, MAX_CHASE_DIST, 20),
		ChaseAndAttack(self.inst, MAX_CHASE_TIME, MAX_CHASE_DIST, 500, KillChester),
		IfNode(function() return StartChoppingCondition(self.inst) end, "chop", 
                WhileNode(function() return KeepChoppingAction(self.inst) end, "keep chopping",
                    LoopNode{ 
                            DoAction(self.inst, FindTreeToChopAction )})),
		IfNode(function() return StartMiningCondition(self.inst) end, "mine", 
                WhileNode(function() return KeepChoppingAction(self.inst) end, "keep mining",
                    LoopNode{ 
                            DoAction(self.inst, FindRockToMineAction )})),
		IfNode(function() return StartHackingCondition(self.inst) end, "hack", 
				WhileNode(function() return KeepChoppingAction(self.inst) end, "keep hacking",
					LoopNode{ 
							DoAction(self.inst, FindVineToHackAction )})),

		WhileNode(function() return self.inst.components.combat.target == GetPlayer() or self.inst.components.combat.target == nil end, "Should Pick",
            DoAction(self.inst, TryToInstaTake)), -- alawaldo
        IfNode( function() return self.inst.components.combat.target == nil and WallInWay(self.inst, self) and math.random() <0.25 end, "hasnotarget", AttackWall(self.inst)),
		Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
		IfNode(function() return math.random() <0.1 end, "net", 
                WhileNode(function() return KeepChoppingAction(self.inst) end, "keep snapping",
                    LoopNode{ 
                            DoAction(self.inst, SnapAtInsects )})), --edit for light from fireflies
		IfNode(function() return  (math.random() < 0.0001 or (self.inst.components.container:IsFull() and math.random() <0.01)) and IsPlayerUsingItem(self.inst, "hat_wizzard") end, "steal hat_wizzard",  --steal hat code  
                WhileNode(function() return KeepChoppingAction(self.inst) end, "keep snapping",
                    LoopNode{ 
                            DoAction(self.inst, FindTheHatAndStealIt )})),--BufferedAction(self.inst, IsPlayerUsingItem(self.inst, "hat_wizzard"), ACTIONS.STEAL)
        FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
        Wander(self.inst, function() return self.inst.components.knownlocations:GetLocation("home") end, MAX_WANDER_DIST),
		StandStill(self.inst, function() return self.inst.sg:HasStateTag("idle") end, nil),
        
    }, .25)
    self.bt = BT(self.inst, root)
end

return LuggageBrain
