
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {
        Asset( "ANIM", "anim/rnw.zip" ),
	--Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
}
local prefabs = {}

local start_inv =
{
    "hat_wizzard",
}


local SPAWN_DIST = 30

local function GetSpawnPoint(pt)

    local theta = math.random() * 2 * PI
    local radius = SPAWN_DIST

	local offset = FindWalkableOffset(pt, theta, radius, 12, true)
	if offset then
		return pt+offset
	end
end

local function SpawnLuggage(inst)
    local pt = Vector3(inst.Transform:GetWorldPosition())
        
    local spawn_pt = GetSpawnPoint(pt)
    if spawn_pt then
        --trace("    at", spawn_pt)
        local luggage = SpawnPrefab("rnw_luggage")
        if luggage then
            luggage.Physics:Teleport(spawn_pt:Get())
            luggage:FacePoint(pt)
            return luggage
        end
    else
        -- this is not fatal, they can try again in a new location by picking up the bone again
        --trace("luggage_eyebone - SpawnLuggage: Couldn't find a suitable spawn point for luggage")
    end
end

local function RebindLuggage(inst, luggage)
    luggage = luggage or TheSim:FindFirstEntityWithTag("luggage")
    if luggage then
        inst.AnimState:PlayAnimation("idle_loop", true)
        luggage.components.follower:SetLeader(inst)
        return true
    end
end

local function RespawnLuggage(inst)
    local luggage = TheSim:FindFirstEntityWithTag("luggage")
    if not luggage then
        luggage = SpawnLuggage(inst)
    end
    RebindLuggage(inst, luggage)
end

local INITIAL_HEALTH = 200
local PREV_HEALTH = 0
local function OnHurt(inst)
	--print("inst on hurt", inst.components.health.currenthealth )
	local delta_hp =  inst.components.health.currenthealth - PREV_HEALTH
	if  delta_hp < 0 then
		inst.components.sanity:DoDelta(delta_hp,0)
		inst.hpdown = true
		--inst.components.talker:Say(STRINGS.CHARACTERS.RNW.fear)
		--inst:DoTaskInTime(0,inst.components.talker:Say(STRINGS.CHARACTERS.RNW.fear))
	else
		inst.hpdown = false
	end
	PREV_HEALTH = inst.components.health.currenthealth
end

local function VerifyHat(inst)
	-- check that the hat exists and is accessible
	if inst.components.inventory:Has("hat_wizzard", 1) then return end
	for k, v in pairs (inst.components.inventory.equipslots) do
		if v.prefab == "hat_wizzard" then
			return
		end
	end 
	local wizz_hat = TheSim:FindFirstEntityWithTag("hat_wizzard")
	local luggage = TheSim:FindFirstEntityWithTag("luggage")
	if not luggage then return end
	local pt = Vector3(luggage.Transform:GetWorldPosition())
	if wizz_hat then
		if not wizz_hat:IsOnValidGround() then
			print("cant reach hat remove and respawn!")
			wizz_hat:Remove()
			wizz_hat= false
		end
		if luggage.components.container:Has("hat_wizzard",1) then
			luggage.components.container:DropItem(wizz_hat, true, true)
		end
		--print("is it in luggage?", luggage.components.inventory:GetItemSlot("hat_wizzard") )
		
	end
	if not wizz_hat then
		local wizz_hat = SpawnPrefab("hat_wizzard")
		if wizz_hat then
			luggage.components.inventory:GiveItem(wizz_hat)
			--wizz_hat.Physics:Teleport(pt)
		else
			print("BUGBUGEBUGE HAT NOT SPAWNING")
		end
		
	end
end

local fn = function(inst)
	
	-- choose which sounds this character will play
	inst.soundsname = "wolfgang"

	-- a minimap icon must be specified
	inst.MiniMapEntity:SetIcon("rincewind.tex")

	-- todo: Add an example special power here.
	inst.components.health:SetMaxHealth(INITIAL_HEALTH)
	inst.components.sanity:SetMax(TUNING.WILSON_SANITY * 0.5)
	inst.components.sanity.dapperness = -3.9
	--print("moisture delta",self:GetMoistureDelta())
	
	--inst:AddComponent("combat")
	--inst.components.combat:SetTarget(nil)
    inst.components.combat.damagemultiplier = 0.05 --TUNING.WENDY_DAMAGE_MULT

	-----------------------------
	if IsDLCEnabled(REIGN_OF_GIANTS) then 
		inst.components.temperature.overheattemp = 90
	end
	
	-----------------------------

	inst:AddComponent("leader")
	inst:AddComponent("talker") --get rnw to talk
	inst.hpdown = false
	inst:ListenForEvent("healthdelta", OnHurt) 
	inst:ListenForEvent("sanitydelta", VerifyHat) 
	
	inst:AddTag("rincewind")
	inst.fixtask = inst:DoTaskInTime(0, function() RespawnLuggage(inst) end)

end


-- strings! Any "WOD" below would have to be replaced by the prefab name of your character.

-- First up, the character select screen lines 
-- note: these are lower-case character name
STRINGS.CHARACTER_TITLES.rnw = "Rincewind"
STRINGS.CHARACTER_NAMES.rnw = "Rincewind"
STRINGS.CHARACTER_DESCRIPTIONS.rnw = "*Has a nice hat that says WIZZARD on the front\n*Is accompanied by the Luggage"
STRINGS.CHARACTER_QUOTES.rnw = "\"It's not paranoia if they really are out to get you.\""

-- You can also add any kind of custom dialogue that you would like. Don't forget to make
-- categores that don't exist yet using = {}
-- note: these are UPPER-CASE charcacter name
STRINGS.CHARACTERS.RNW = require "speech_rincewind"

--STRINGS.CHARACTERS.RNW = {}
--STRINGS.CHARACTERS.RNW.DESCRIBE = {}
STRINGS.CHARACTERS.RNW.DESCRIBE.EVERGREEN = "Those pine needles look awfully sharp."
STRINGS.CHARACTERS.RNW.DESCRIBE.RABBIT = "Lucky bastards, they can hide in their burrows."
STRINGS.CHARACTERS.RNW.DESCRIBE.CROW = "Urgh, it feeds off carrion."
STRINGS.CHARACTERS.RNW.DESCRIBE.SKELETON = "I better leave before DEATH comes for him."
STRINGS.CHARACTERS.RNW.DESCRIBE.POND = "Creatures of the deep sleep. Don't disturb the waters."
STRINGS.CHARACTERS.RNW.DESCRIBE.SPIDER = "Argh! Between 7 and 9 legs. Not good."
STRINGS.CHARACTERS.RNW.DESCRIBE.BEEFALO = "Those horns look dangerous."
STRINGS.CHARACTERS.RNW.DESCRIBE.TENTACLE = "Bel-Shamharoth! Argh!"
STRINGS.CHARACTERS.RNW.DESCRIBE.RABBITHOLE = "Wish I could hide there."
STRINGS.CHARACTERS.RNW.DESCRIBE.PIGKING = "What a hideous... uhh... Your majesty."
STRINGS.CHARACTERS.RNW.DESCRIBE.EYEPLANT = "Don't eat me!"
STRINGS.CHARACTERS.RNW.DESCRIBE.DEERCLOPS = "Don't hurt me!"
STRINGS.CHARACTERS.RNW.ANNOUNCE_WORMHOLE = "Never again."
STRINGS.CHARACTERS.RNW.ANNOUNCE_TORCH_OUT = "I hope they can't see me too."
STRINGS.CHARACTERS.RNW.ANNOUNCE_NODANGERSLEEP = "Sleep? Now? Are you nuts? RUN!"
STRINGS.CHARACTERS.RNW.ANNOUNCE_INV_FULL = "Hmm, I need some type of walking container..."
STRINGS.CHARACTERS.RNW.ANNOUNCE_ENTER_DARK = "Maybe they won't see me. I can't see me!"
STRINGS.CHARACTERS.RNW.ANNOUNCE_DUSK = "Time to find somewhere safe."
STRINGS.CHARACTERS.RNW.ANNOUNCE_DEERCLOPS = "The gods were running out of ideas when they made that!"
STRINGS.CHARACTERS.RNW.ANNOUNCE_COLD = "Hopefully the monsters will freeze to death."
STRINGS.CHARACTERS.RNW.ANNOUNCE_CHARLIE = "That's not a good sound."
STRINGS.CHARACTERS.RNW.ANNOUNCE_CHARLIE_ATTACK = "Aaarrrghh!"
STRINGS.CHARACTERS.RNW.ANNOUNCE_NODAYSLEEP = "I should travel while it is less dark."
STRINGS.CHARACTERS.RNW.ANNOUNCE_HOUNDS = "Luggage! Help!"

return MakePlayerCharacter("rnw", prefabs, assets, fn, start_inv)
