require "prefabutil"
local brain = require "brains/luggagebrain"
require "components/container"
require "components/thief"
local PersistentData = require "persistentdata"
--require "saveindex"

local WAKE_TO_FOLLOW_DISTANCE = 7
local SLEEP_NEAR_LEADER_DISTANCE = 7
local MAX_TARGET_SHARES = 5
local SHARE_TARGET_DIST = 30
local INTENSITY = .8
local last_target = "banana"
local damage_mult = 0 
local FIREFLYINTENSITY = 0.0099
local FIREFLYRADIUS = 0.05
local FIREFLYDROPOFF = 0.5
--------------
local assets =
{
	--Asset("ANIM", "anim/chester.zip"),
	Asset("ANIM", "anim/rnw_luggage.zip"),
	Asset("ANIM", "anim/rnw_luggage_water.zip"),
	Asset("ANIM", "anim/ui_chest_3x2.zip"),
	Asset("ATLAS", "images/inventoryimages/container.xml"),
	Asset("SOUND", "sound/chester.fsb"),
	
}
----------------
local prefabs =
{
	--"chester_eyebone",
	"die_fx"
}

---------------------
local function SaveLuggInv(dirty, inst)
	local lugg_contents = {}
	for k,v in pairs(inst.components.container.slots) do
		if v:IsValid() then --only save the valid items
			lugg_contents[k] = v:GetSaveRecord()
		end
	end
	--for k,v in pairs(lugg_contents) do print ("saving", k, v, "<-------") end
	inst.lugg_inv:SetValue(inst.lugg_inv_save_slot, lugg_contents)
	inst.lugg_inv:Save(dirty)
end

local function LoadLuggInv(inst)
	print("attempting to load lugg")
	inst.lugg_inv:Load()
	local lugg_contents = inst.lugg_inv:GetValue(inst.lugg_inv_save_slot)
	--for k,v in pairs(lugg_contents) do print ("found", k, v, "<-------") end
	local game_data = SaveGameIndex:GetModeData(SaveGameIndex.current_slot, SaveGameIndex.data.slots[SaveGameIndex.current_slot].current_mode)
	if game_data.day == 1 and game_data.world == 1 or game_data.world == nil then 
		print("New game detected, no luggage carry over")
	elseif lugg_contents then 
		for k,v in pairs(lugg_contents) do
			local curr_item = SpawnSaveRecord(v, inst)
			if curr_item then
				inst.components.container:GiveItem(curr_item, k)
			end
		end
	end
end

local function OnNewSpawn(inst)
	if #inst.components.container.slots == 0 then
		LoadLuggInv(inst)
	else 
		print("luggage was not empty on new spawn?")
	end
	-- refill luggage assert he is empty...
end

local function OnAttacked(inst, data)
	--print(inst, "OnAttacked")
	--print("ATTACKED")
	--prevent attacking player bees
	local attacker = data.attacker
	if attacker.prefab == "bee" and attacker.components.homeseeker and attacker.components.homeseeker.home.prefab == "beebox" then inst.components.combat:SetTarget(nil)
	else inst.components.combat:SetTarget(attacker)
	end 
	-- give luggage default damage for a new target
	inst.components.combat.defaultdamage = TUNING.PIG_DAMAGE*3
	--
end

local function OnAttackOther(inst, data)
	--print("sharetarget", data.target)
	inst.components.combat:ShareTarget(data.target, SHARE_TARGET_DIST, function(dude) return dude:HasTag("luggage") or dude:HasTag("companion") and not dude.components.health:IsDead() and not dude:HasTag("gnat") end, 5)
	-- give luggage increasing damage for repeated attacks
	if data.target:HasTag("gnat") then
		--print("fuck gnats")
		inst.components.combat:ExcludeTarget(data.target)
		--inst.components.combat:SetTarget(nil)
	end
		
	if data.target == last_target then
		damage_mult = damage_mult + inst.components.combat.defaultdamage*3
		inst.components.combat.defaultdamage = damage_mult
		--print("now punching", inst.components.combat.defaultdamage)
	else
		last_target = data.target
		damage_mult = 0
		inst.components.combat.defaultdamage = TUNING.PIG_DAMAGE*3
		if data.target:HasTag("chester") then
			inst.components.combat.defaultdamage = 200
		end
	end
end

local function RincewindAttacked(inst, data)
	inst.components.combat:ShareTarget(data.target, SHARE_TARGET_DIST, function(dude) return dude:HasTag("luggage") or dude:HasTag("companion") and not dude.components.health:IsDead() and not dude:HasTag("gnat") end, 5)
end

local function Retarget(inst)

	local newtarget = FindEntity(inst, 20, function(guy)
			return  guy.components.combat and 
					inst.components.combat:CanTarget(guy) and
					(guy.components.combat.target == GetPlayer() or GetPlayer().components.combat.target == guy)
	end)
	-- give luggage default damage for a new target
	inst.components.combat.defaultdamage = TUNING.PIG_DAMAGE*3
	--print("normal")
	--
	return newtarget
end

local function NormalKeepTargetFn(inst, target)
	-- give luggage huge increasing damage for a current target
	inst.components.combat.defaultdamage = 2000
	--print("falconpunch")
	--
	--give up on dead guys
	return inst.components.combat:CanTarget(target)
end


----------------------


local function ShouldWakeUp(inst)
	return DefaultWakeTest(inst) or not inst.components.follower:IsNearLeader(WAKE_TO_FOLLOW_DISTANCE)
end


local function OnOpen(inst)
	if inst.first_check_after_spawn then
		inst.first_check_after_spawn = false
		OnNewSpawn(inst)
	end
	if not inst.components.health:IsDead() then
		--inst.sg:GoToState("open") --crashes the game through 'sleep' event if enabled
		inst.sg:GoToState("open_idle") -- bypass above and directly call open_idle
	end
end 

local function OnClose(inst) 
	SaveLuggInv(true, inst)
	if not inst.components.health:IsDead() then
		inst.sg:GoToState("close")
	end
end 


local function flicker_update(inst)
		local time = GetTime()*30
		local flicker = ( math.sin( time ) + math.sin( time + 2 ) + math.sin( time + 0.7777 ) ) / 2.0 -- range = [-1 , 1]
		flicker = ( 1.0 + flicker ) / 2.0 -- range = 0:1
		inst.Light:SetRadius( 1.5 + 0.1 * flicker)
		inst.flickertask = inst:DoTaskInTime(0.1, function() flicker_update(inst) end)
end

local function fade_in(inst)
		inst.components.fader:StopAll()
		inst.Light:Enable(true)
		--flicker_stop(inst)
		flicker_update(inst)
		inst.Light:SetFalloff(.5)
		inst.Light:SetIntensity(INTENSITY)
		inst.Light:SetRadius(1.5)
		inst.Light:SetColour(125/255, 249/255, 255/255)
		inst.components.fader:Fade(0, INTENSITY, 5*FRAMES, function(v) inst.Light:SetIntensity(v) end)
end

local function discharge(inst)
	inst.AnimState:SetBloomEffectHandle("")
	inst.charged = false
	inst.chargeleft = nil
	inst.Light:Enable(false)
	if inst.zaptask then
		inst.zaptask:Cancel()
		inst.zaptask = nil
	end
end

local function dozap(inst)
	if inst.zaptask then
		inst.zaptask:Cancel()
		inst.zaptask = nil
	end
	inst.SoundEmitter:PlaySound("dontstarve/common/lightningrod")
	--small_puff
	local fx = SpawnPrefab("sparks")
	local pos = inst:GetPosition()
	fx.Transform:SetPosition(pos.x, pos.y, pos.z)
	local fx = SpawnPrefab("sparks")
	local pos = inst:GetPosition()
	fx.Transform:SetPosition(pos.x, pos.y, pos.z)
	local fx = SpawnPrefab("explode_small")
	local pos = inst:GetPosition()
	fx.Transform:SetPosition(pos.x, pos.y, pos.z)
	local fx = SpawnPrefab("small_puff")
	local pos = inst:GetPosition()
	fx.Transform:SetPosition(pos.x, pos.y, pos.z)
	
	--PlayFX(Vector3(inst.Transform:GetWorldPosition() ), "lightning_rod_fx", "lightning_rod_fx", "idle")
	inst.zaptask = inst:DoTaskInTime(math.random(10, 30), dozap)
end

local function setcharged(inst)
	dozap(inst)
	--inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
	--inst.Light:Enable(true)
	fade_in(inst)
	inst.charged = true
	inst.chargeleft = 3
	inst:ListenForEvent( "daycomplete", function()
		if inst.chargeleft then
			dozap(inst)
			inst.chargeleft = inst.chargeleft - 1
			if inst.chargeleft <= 0 then
				discharge(inst)
			end
		end
	end, GetWorld())
end

local function onlightning(inst)
	--onhit(inst)
	setcharged(inst)
end

local function OnSave(inst, data)
	if inst.charged then
		data.charged = inst.charged
		data.chargeleft = inst.chargeleft
	end
	data.first_check_after_spawn = inst.first_check_after_spawn
	return data
	
end

local function OnLoad(inst, data)
	if data and data.charged and data.chargeleft then
		setcharged(inst)
		inst.chargeleft = data.chargeleft
	end
	inst:FireFlyLight(inst)
	inst.first_check_after_spawn = data.first_check_after_spawn
end

local slotpos = {}
	
for y = 3, 0, -1 do
	for x = 0, 3 do
		table.insert(slotpos, Vector3(80*x-80*2+40, 80*y-80*2+40,0))
	end
end


--firefly light
local function FireFlyLight(inst)
	--print("Checking Fireflies")
	local has_firefly, nr_fireflies = inst.components.container:Has("fireflies",0)
	local has_lightbulbs, nr_lightbulbs = inst.components.container:Has("lightbulb",0)
	--print(has_firefly,nr_fireflies, FIREFLYINTENSITY*nr_fireflies)
	local lightmulti = 0
	if not inst.charged then
		inst.components.fader:StopAll()
		inst.Light:Enable(true)
		--inst.Light:SetIntensity(0) 
		if nr_fireflies +nr_lightbulbs > 100 then 
			lightmulti = 100 
		else lightmulti = nr_fireflies +nr_lightbulbs
		end
		lightmulti = lightmulti/2.5 --major adjust parameter
		inst.Light:SetFalloff(1-lightmulti*(1-FIREFLYDROPOFF)*0.005)
		inst.Light:SetRadius(FIREFLYRADIUS*lightmulti)
		inst.Light:SetColour(180/255, 195/255, 150/255)
		--print(nr_fireflies, lightmulti, FIREFLYINTENSITY*lightmulti, "dropoff", 1-lightmulti*(1-FIREFLYDROPOFF)*0.01, "rad",  FIREFLYRADIUS*lightmulti)
		--inst.components.fader:Fade(0, FIREFLYINTENSITY*nr_fireflies, 3+math.random()*2, function(v) inst.Light:SetIntensity(v) end)
		inst.components.fader:Fade(0, FIREFLYINTENSITY*lightmulti, 5*FRAMES, function(v) inst.Light:SetIntensity(v) end)
	else print("charged with lightning - the fireflies are afraid!")
	end
end

local function OnWaterChange(inst, onwater)
	--inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/crocodog/emerge")

	if onwater then
		if inst.DynamicShadow then
			inst.DynamicShadow:Enable(false)
		end
		inst.AnimState:SetBank("rnw_luggage_water")
		inst.AnimState:SetBuild("rnw_luggage_water")
		inst.components.rowboatwakespawner:StartSpawning()
	else
		if inst.DynamicShadow then
			inst.DynamicShadow:Enable(true)
		end
		inst.AnimState:SetBank("rnw_luggage")
		inst.AnimState:SetBuild("rnw_luggage")
		inst.components.rowboatwakespawner:StopSpawning()
	end
	
	local splash = SpawnPrefab("splash_water")
	local ent_pos = Vector3(inst.Transform:GetWorldPosition())
	splash.Transform:SetPosition(ent_pos.x, ent_pos.y, ent_pos.z)
	
	if inst.sg then
		inst.sg:GoToState("idle")
	end
end

local function IsValidTarget(target)
    if not target 
	   or not target:IsValid()
       or not target.components
       or not target.components.combat
       or not target.entity:IsVisible()
       or not target.components.health
       or target == self.inst
	   or target.HasTag("gnat")
       or target.components.health:IsDead()
       or (target:HasTag("shadow") and not self.inst.components.sanity)
       or Vector3(target.Transform:GetWorldPosition()).y > self.attackrange then
        return false
    else
		return true
	end
end


local function create_luggage(Sim)
	--print("Luggage! - create_luggage")
	
	if TheSim:FindFirstEntityWithTag("luggage") then 
		--print("making a duplicate luggage, dont do that")
		return TheSim:FindFirstEntityWithTag("luggage") 
	end
	
	local inst = CreateEntity()
	
	inst:AddTag("companion")
	inst:AddTag("character") -- mod
	inst:AddTag("scarytoprey") -- mod
	inst:AddTag("luggage")
	inst:AddTag("FX")
	inst:AddTag("notraptrigger")
	

	inst.entity:AddTransform()
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("rnw_luggage.tex")
	
	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor.runspeed = TUNING.PIG_RUN_SPEED*9
	inst.components.locomotor.walkspeed = TUNING.PIG_WALK_SPEED*3
	
	inst:AddComponent("worker")
	inst.components.worker:SetAction(ACTIONS.CHOP,10)
	inst.components.worker:SetAction(ACTIONS.MINE,10)
	inst.components.worker:SetAction(ACTIONS.DIG,10)
	
	
	inst:AddComponent("combat")
	inst.components.combat:SetTarget(nil)
	inst.components.combat.defaultdamage = TUNING.PIG_DAMAGE*3
	inst.components.combat.playerdamagepercent = TUNING.ABIGAIL_DMG_PLAYER_PERCENT
	inst.components.combat:SetAttackPeriod(TUNING.PIG_ATTACK_PERIOD*0.5)
	inst.components.combat:SetRetargetFunction(3, Retarget)
	inst:ListenForEvent("attacked", OnAttacked)	
	inst:ListenForEvent("onattackother", OnAttackOther)
	inst.components.combat.hiteffectsymbol = "chester_body"
	
	--print("   AnimState")
	inst.entity:AddAnimState()
	inst.AnimState:SetBank("rnw_luggage")
	inst.AnimState:SetBuild("rnw_luggage")

	--print("   sound")
	inst.entity:AddSoundEmitter()

	--print("   shadow")
	inst.entity:AddDynamicShadow()
	inst.DynamicShadow:SetSize( 2, 1.5 )

	--print("   Physics")
	MakeCharacterPhysics(inst, 75, .75)
	
	--print("   Collision")
	inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
	inst.Physics:ClearCollisionMask()
	if IsDLCEnabled(PORKLAND_DLC) then 
		inst.Physics:CollidesWith(GetWorldCollision())
	else
		inst.Physics:CollidesWith(COLLISION.WORLD)
	end
	inst.Physics:CollidesWith(COLLISION.OBSTACLES)
	inst.Physics:CollidesWith(COLLISION.CHARACTERS)

	inst.Transform:SetFourFaced()

	
	-- MOD set lighting
	inst:AddComponent("fader")
	local light = inst.entity:AddLight()
	light:SetFalloff(.5)
	light:SetIntensity(INTENSITY)
	light:SetRadius(1.5)
	light:Enable(false)
	light:SetColour(200/255, 200/255, 245/255)
	
	--fade_in(inst)
	-- Make Luggage a Lightning rod
	inst:AddTag("lightningrod")
	inst:ListenForEvent("lightningstrike", onlightning)
	inst.lightningpriority = 0
	
	-- /MOD
	inst:AddComponent("thief")
	
	------------------------------------------

	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(TUNING.CHESTER_HEALTH*500)
	inst.components.health:StartRegen(TUNING.CHESTER_HEALTH_REGEN_AMOUNT*50, TUNING.CHESTER_HEALTH_REGEN_PERIOD)
	inst:AddTag("noauradamage")
	--inst.components.health.invincible = true --mod

	--print("   inspectable")
	inst:AddComponent("inspectable")
	inst.components.inspectable:RecordViews()
	----inst.components.inspectable.getstatus = GetStatus

	--print("   follower")
	inst:AddComponent("follower")
	inst:AddComponent("leader") --allows chester to follow him
	inst:AddTag("companion") 

	--print("   knownlocations")
	inst:AddComponent("knownlocations")

	--print("   burnable")
	MakeSmallBurnableCharacter(inst, "chester_body")
	
	--print("   inventory")
	--inst:AddComponent("inventory")
	
	inst.lugg_inv_save_slot = "lugg_inv_" .. SaveGameIndex.current_slot
	--print(inst.lugg_inv_save_slot, "<----------------")
	inst.lugg_inv = PersistentData(inst.lugg_inv_save_slot)
	
	
	inst:AddComponent("container")
	inst:AddTag("fridge")
	inst.components.container.is_luggage = true

	inst.components.container.onopenfn = OnOpen
	inst.components.container.onclosefn = OnClose
	
	inst.components.container.widgetanimbank = nil
	inst.components.container.widgetanimbuild = nil
	inst.components.container.widgetbgatlas = "images/inventoryimages/container.xml"
	inst.components.container.widgetbgimage = "container.tex"
	inst.components.container.widgetpos = Vector3(0,-200,0)
	inst.components.container.numslots = #slotpos
	inst.components.container.widgetslotpos = slotpos
	inst.components.container.side_align_tip = 160 --new

	----------------------------
	-- Addons for DLC
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then 
		MakeAmphibiousCharacterPhysics(inst, 75, 1.25)
		inst:AddTag("amphibious")
		inst.components.worker:SetAction(ACTIONS.HACK,10)
		inst.components.follower:SetFollowExitDestinations({EXIT_DESTINATION.LAND,EXIT_DESTINATION.WATER})
		inst:AddComponent("tiletracker")
		inst.components.tiletracker:SetOnWaterChangeFn(OnWaterChange)
		inst.components.tiletracker:Start()
		inst:AddComponent("rowboatwakespawner")
	--elseif IsDLCEnabled(REIGN_OF_GIANTS) then 
		-- code here
		--end
	end
	----------------------------
	
	--print("   sg")
	inst:SetStateGraph("SGLuggage")
	inst.sg:GoToState("idle")
	
	
	--inst.persists = true
	inst.first_check_after_spawn = true --flag to check for a previous contents in lugg_inv on first spawn
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	--print("   brain")
	inst:SetBrain(brain)	
	
	inst.FireFlyLight = FireFlyLight

	return inst
end

return Prefab( "common/rnw_luggage", create_luggage, assets, prefabs)
